/* hashtable.h
 *
 * Copyright 2019 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#pragma once

#include "slist.h"
#include "types.h"

typedef struct _HashTable HashTable;


HashTable *  hash_table_new     (HashFunc      hash_func,
                                 EqualFunc     equal_func,
                                 FreeFunc      free_key_func,
                                 FreeFunc      free_value_func);
void         hash_table_free    (HashTable    *self);

void         hash_table_insert  (HashTable  *table,
                                 const void *key,
                                 const void *value);
const void * hash_table_lookup  (HashTable  *self,
                                 const void *key);

void         hash_table_foreach (HashTable   *self,
                                 HashIterFunc func,
                                 void        *user_data);
